package com.aliyun.odps.spark.examples.makedata_ad.v20240718

import com.alibaba.fastjson.{JSON, JSONObject}
import com.aliyun.odps.TableSchema
import com.aliyun.odps.data.Record
import com.aliyun.odps.spark.examples.myUtils.env
import org.apache.spark.rdd.RDD
import org.apache.spark.sql.SparkSession

import scala.collection.JavaConverters.asScalaSetConverter
import scala.collection.mutable


/*
   diff data
 */

object diff_data_20250422 {
  def main(args: Array[String]): Unit = {
    val spark = SparkSession
      .builder()
      .appName(this.getClass.getName)
      .getOrCreate()
    val sc = spark.sparkContext

    // 2 读取odps+表信息
    val odpsOps = env.getODPS(sc)
    val odpsData1 = odpsOps.readTable(project = "loghubods",
      table = "ad_easyrec_train_data_v3_sampled",
      partition = "dt=20250422",
      transfer = func,
      numPartition = 64)

    val odpsData2 = odpsOps.readTable(project = "loghubods",
      table = "ad_engine_statistics_log_per_linshi_dt",
      partition = "dt=20250422",
      transfer = func1,
      numPartition = 64).map(record => {
      val pqtid = record.getString("pqtid")
      val allfeature: JSONObject = if (record.isNull("allfeature")) new JSONObject() else
        JSON.parseObject(record.getString("allfeature"))
      allfeature.put("pqtid", pqtid)
      allfeature
    })

    val rdd1Pairs: RDD[(String, JSONObject)] = odpsData1.map(map => (map.getString("pqtid"), map))
    val rdd2Pairs: RDD[(String, JSONObject)] = odpsData2.map(map => (map.getString("pqtid"), map))


    // 进行 join 操作
    val joinedRDD: RDD[(String, (JSONObject, JSONObject))] = rdd2Pairs.join(rdd1Pairs)
    println("joinedRDD count", joinedRDD.count)

    // 使用 map 操作生成键值对
    val diffPairs = joinedRDD.flatMap { case (_, (map1, map2)) =>
      val keys = map1.keySet().asScala.toSet ++ map2.keySet().asScala.toSet
      keys.flatMap { key =>
        if (map1.containsKey(key) && map2.containsKey(key)) {
          val value1 = map1.getString(key)
          val value2 = map2.getString(key)
          (tryToNumber(value1), tryToNumber(value2)) match {
            case (Some(num1), Some(num2)) =>
              val diff = math.abs(num1.doubleValue() - num2.doubleValue())
              if (diff > 0) {
                Seq((key, (diff, num1.doubleValue(), num2.doubleValue(), 1)))
              } else {
                Seq.empty
              }
            case _ =>
              val str1 = if (value1 != null) value1 else ""
              val str2 = if (value2 != null) value2 else ""
              if (str1 != str2) {
                Seq((key, (0.0, 0.0, 0.0, 1)))
              } else {
                Seq.empty
              }
          }
        } else {
          Seq.empty
        }
      }
    }

    // 使用 reduceByKey 进行聚合
    val aggregatedPairs = diffPairs.reduceByKey { case ((diffSum1, onlineSum1, offlineSum1, count1), (diffSum2, onlineSum2, offlineSum2, count2)) =>
      (diffSum1 + diffSum2, onlineSum1 + onlineSum2, offlineSum1 + offlineSum2, count1 + count2)
    }

    // 收集结果
    val result = aggregatedPairs.collect()

    // 处理结果
    val featureDiffSum = mutable.Map[String, Double]()
    val featureOnlineSum = mutable.Map[String, Double]()
    val featureOfflineSum = mutable.Map[String, Double]()
    val featureCount = mutable.Map[String, Int]()

    result.foreach { case (key, (diffSum, onlineSum, offlineSum, count)) =>
      featureDiffSum(key) = diffSum
      featureOnlineSum(key) = onlineSum
      featureOfflineSum(key) = offlineSum
      featureCount(key) = count
    }

    // 打印结果
//    println("featureDiffSum: " + featureDiffSum)
//    println("featureCount: " + featureCount)


    val count2 = joinedRDD.count()
    println("count2:" + count2)
    // 输出每个特征的平均差异
    println("每个特征的平均差异:")
    println(featureDiffSum.size)
    featureDiffSum.foreach { case (feature, sum) =>
      val count = featureCount(feature)
      val onlineSum = featureOnlineSum(feature)
      val offlineSum = featureOfflineSum(feature)
      val averageDiff = sum / count
      val averageOnline = onlineSum / count
      val averageOffline = offlineSum / count
      println(s"$feature,$averageDiff,$averageOnline,$averageOffline,$count")
    }
  }

  def tryToNumber(value: Any): Option[java.lang.Number] = {
    value match {
      case num: java.lang.Number => Some(num)
      case str: String =>
        try {
          Some(str.toDouble)
        } catch {
          case _: NumberFormatException => None
        }
      case _ => None
    }

  }

  def func(record: Record, schema: TableSchema): JSONObject = {
    val featureMap = new JSONObject()
    val columns = schema.getColumns
    for (i <- 0 until columns.size()) {
      val column = columns.get(i)
      val name = column.getName
      var key = name.replace("_x_", "*").replace("_view", "(view)")
      val value = record.get(name)
      value match {
        case byteArray: Array[Byte] =>
          try {
            // 将字节数组转换为字符串，使用 UTF-8 编码
            val str = new String(byteArray, "UTF-8")
            featureMap.put(key, str)
          } catch {
            case e: Exception =>
              println(s"转换过程中出现错误: ${e.getMessage}")
          }
        case num: java.lang.Number =>
          // 数字类型转换为字符串
          val str = num.toString
          featureMap.put(key, str)
        case _ =>
          featureMap.put(key, record.get(name))
      }
    }
    featureMap
  }

  def func1(record: Record, schema: TableSchema): Record = {
    record
  }


}
